/*
    interrupt.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   interrupt.c

- Version               :   1.0.0
- Date                  :   May 2019
- Developer             :   Milos Vidojevic

    Common interrupt module.

    Hints :

    - Copy file inside [project base directory]/portable.
    - Replace all "---" appearances with proper values.

*/
/* -------------------------------------------------------------------------- */

#include "simplelink.h"

#if ( _MMB_MODULE_WIFI == 1 )

/*
    WiFi interrupt related functions and variables.
*/

P_EVENT_HANDLER wifi_isr_event;

void bsp_wifi_driver_interrupt_enable ( void )
{
    //  Enable interrupt vector for CC3100 INT pin.

    NVIC_IntEnable( IVT_INT_GPIOX );         // Enable GPIOB interrupt
}

void bsp_wifi_driver_interrupt_disable ( void )
{
    //  Disable interrupt vector for CC3100 INT pin.

    NVIC_IntDisable( IVT_INT_GPIOX );        // Disable GPIOB interrupt
}

/*
    WiFi ISR Function.
*/

void WIFI_EXT_ISR ( void ) iv IVT_INT_GPIOX ics ICS_AUTO
{
    //  Reset interrupt flag
    
    GPIO_PORTX_AHB_ICR = _GPIO_PINMASK_X;        // clear flag

    if ( wifi_isr_event != ( void * ) 0 )
    {
        ( *wifi_isr_event )( 0 );
    }
}

#endif

/*
    USB interrupt related functions and variables.
*/

#if ( _MMB_MODULE_USB == 1 )

    #ifdef TEST_PROJECT

        void USB_ISR( ) iv IVT_INT_USB
        {
            USBHost_IntHandler( );
        }
    
    #endif

#endif

/*
    Common initialize functions which should initialize all necessary 
    interrupts.
*/

void interrupts_enable ( void )
{
    /*
        Initialize external interrupt on CC3100 INT pin.

        NOTE : 

            Interrupt must be configured to trigger on rising edge.
    */
    
    #if ( _MMB_MODULE_WIFI == 1 )
    
        NVIC_IntDisable( IVT_INT_GPIOX );         // Enable GPIOX interrupt
        DisableInterrupts( );

        GPIO_Digital_Input( &GPIO_PORTB_AHB, _GPIO_PINMASK_2 );

        GPIO_PORTX_AHB_AFSEL = _GPIO_PINMASK_X;   // Enable alternate pin functions
        GPIO_PORTX_AHB_IS    = 0;                 // Edge sensitive
        GPIO_PORTX_AHB_IBE   = 0;                 // Edge/Level detection triggers interrupt
        GPIO_PORTX_AHB_IEV   = _GPIO_PINMASK_X;   // Rising Edge will trigger interrupt
        GPIO_PORTX_AHB_IM    = _GPIO_PINMASK_X;   // Set mask to pinX
        NVIC_IntEnable( IVT_INT_GPIOX );          // Enable GPIOX interrupt

    #endif

    /*
        Initialize global interrupts and enable vectors.

            - Enable interrupt vector for USB.
            - Enable interrupt vector for CC3100 INT pin.
    */
    
    #if ( _MMB_MODULE_USB == 1 )

        #ifdef TEST_PROJECT

            NVIC_IntEnable( IVT_INT_USB );
    
        #endif
    
    #endif
    
    #if ( _MMB_MODULE_WIFI == 1 )

        NVIC_IntEnable( IVT_INT_GPIOB );         // Enable GPIOM interrupt
        
    #endif

    EnableInterrupts( );
}